// :: Part of RegionCreator
// Author: Sunshine

#include "CImageFile.h"


// ** constructor
CImageFile::CImageFile()
{
	this->bFileLoaded = false;
	hBitmap = NULL;

}

// ** destructor
CImageFile::~CImageFile()
{
	this->bFileLoaded = false;
	hBitmap = NULL;

}

// ** Load image file
bool CImageFile::Load(LPSTR filename)
{
	// Use IPicture stuff to use JPG / GIF files
	IPicture* p;
	IStream* s;
	//IPersistStream* ps;
	HGLOBAL hG;
	//void* pp;

	// Read file
	HANDLE hFile = CreateFile(filename, GENERIC_READ,
						FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
		return false;

	int iFileSize = GetFileSize(hFile, NULL);
	hG = GlobalAlloc(GPTR , iFileSize);
	if (!hG)
	{
		CloseHandle(hFile);
		return false;
	}

	DWORD bytesRead;
	ReadFile(hFile, hG, iFileSize, &bytesRead, NULL);
	CloseHandle(hFile);

	// Create an IStream so IPicture can
	CreateStreamOnHGlobal(hG,false,&s);
	if (!s)
	{
		GlobalFree(hG);
		return false;
	}

  OleLoadPicture(s,0,false,IID_IPicture,(void**)&p);

   if (!p)
   {
      s->Release();
      GlobalFree(hG);
      return false;
   }
   s->Release();
   GlobalFree(hG);

   HBITMAP hB = 0;
   p->get_Handle((unsigned int*)&hB);

   // Copy the image. Necessary, because upon p's release,
   // the handle is destroyed.
   hBitmap = (HBITMAP)CopyImage(hB,IMAGE_BITMAP,0,0,
                                    LR_COPYRETURNORG);

   p->Release();
   this->bFileLoaded = true;
   return true;
}



// ** Draw image
bool CImageFile::Draw(HDC hdc, int x, int y)
{

	HDC memDC = CreateCompatibleDC(hdc);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject(memDC, (HBITMAP)hBitmap);

	// Display the bitmap into window
	BITMAP bm;
	GetObject((HBITMAP)hBitmap, sizeof(BITMAP), &bm);
	BitBlt(hdc, x, y, bm.bmWidth, bm.bmHeight, memDC, 0, 0, SRCCOPY);
	// Delete the temporary bitmap DC
	SelectObject(memDC, hOldBitmap);
	DeleteDC(memDC);

	return true;
}

int CImageFile::getHeight()
{
	if (!bFileLoaded) return -1;
	BITMAP bm;
	GetObject((HBITMAP)hBitmap, sizeof(BITMAP), &bm);
	return bm.bmHeight;
}

int CImageFile::getWidth()
{
	if (!bFileLoaded) return -1;
	BITMAP bm;
	GetObject((HBITMAP)hBitmap, sizeof(BITMAP), &bm);
	return bm.bmWidth;
}

// ** Draw part of image
bool CImageFile::DrawEx(HDC hdc, int x_dest, int y_dest, int x_source, int y_source)
{
	HDC memDC = CreateCompatibleDC(hdc);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject(memDC, (HBITMAP)hBitmap);

	// Display the bitmap into window
	BITMAP bm;
	GetObject((HBITMAP)hBitmap, sizeof(BITMAP), &bm);
	BitBlt(hdc, x_dest, y_dest, bm.bmWidth - x_source,
			bm.bmHeight - y_source,
		   memDC, x_source, y_source, SRCCOPY);
	// Delete the temporary bitmap DC
	SelectObject(memDC, hOldBitmap);
	DeleteDC(memDC);

	return true;
}

// ** Draw a part of image with zoom factor
bool CImageFile::DrawExZoom(HDC hdc, int destX, int destY, int destWidth, int destHeight,
				int x_source, int y_source, float zoom)
{

	BITMAP bm;
	GetObject((HBITMAP)hBitmap, sizeof(BITMAP), &bm);

	HDC memDC = CreateCompatibleDC(hdc);
	HBITMAP hOldBitmap = (HBITMAP)SelectObject(memDC, (HBITMAP)hBitmap);
	HDC memDC2 = CreateCompatibleDC(hdc);
	HBITMAP hZoomedBitmap = CreateCompatibleBitmap(hdc, (int)(bm.bmWidth * zoom), (int)(bm.bmHeight * zoom));
	SelectObject(memDC2, (HBITMAP)hZoomedBitmap);

	BOOL retStretch  = StretchBlt(memDC2,
                     0, 0,
                     (int)(bm.bmWidth*zoom), (int)(bm.bmHeight*zoom) ,
                     memDC,
                     0, 0,
                     bm.bmWidth, bm.bmHeight,
                     SRCCOPY);
	DWORD dError = GetLastError();

	int iV = (int)(bm.bmHeight*zoom) - destHeight;
	//int delta = abs(getHeight() - bm.bmHeight);
	iV = abs(iV * (float)((float)(y_source) / (float)(bm.bmHeight - destHeight))) ;

	int iH = (int)(bm.bmWidth*zoom) - destWidth;
	//delta = abs(getWidth() - bm.bmWidth);
	iH = abs(iH * (float)((float)(x_source) / (float)(bm.bmWidth - destWidth))) ;


    BOOL retBlt  =  BitBlt(hdc,
                    0,0,
                    destWidth,
					destHeight,
                    memDC2,
                    iH,iV,
                    SRCCOPY);

    DeleteObject(hZoomedBitmap);
	SelectObject(memDC, hOldBitmap);
	DeleteDC(memDC);
	DeleteDC(memDC2);

	return true;
}




