// :: Part of RegionCreator
// Author: Sunshine

#include "CRegion.h"


// ** constructor
CRegion::CRegion(HBITMAP bitmap)
{
	this->hBitmap = bitmap;
	if (hRegion)
		DeleteObject(hRegion);
	//if (pData)
	//	delete pData;
	width = 0;
	height = 0;
}

CRegion::~CRegion()
{
	if (hRegion)
		DeleteObject(hRegion);
}


bool CRegion::Convert()
{
	BITMAP	bmp;
	GetObject(hBitmap, sizeof(bmp),&bmp);

	// :: convert bitmap to 24bit
	// allocate enough space
	width = bmp.bmWidth;
	width -= (width%4);
	height = bmp.bmHeight;
	buffer = new BYTE[width * height * 3];
	if (!buffer) return false;

	// get desktop device context
	HDC hdc = GetWindowDC(NULL);

	// fill desired structure
	BITMAPINFO bmiInfo;
	bmiInfo.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	bmiInfo.bmiHeader.biWidth = width;
	bmiInfo.bmiHeader.biHeight = -height;
	bmiInfo.bmiHeader.biPlanes = 1;
	bmiInfo.bmiHeader.biBitCount = 24;
	bmiInfo.bmiHeader.biCompression = BI_RGB;
	bmiInfo.bmiHeader.biSizeImage = width * height * 3;
	bmiInfo.bmiHeader.biXPelsPerMeter = 0;
	bmiInfo.bmiHeader.biYPelsPerMeter = 0;
	bmiInfo.bmiHeader.biClrUsed = 0;
	bmiInfo.bmiHeader.biClrImportant = 0;

	// get pixels from the original bitmap converted to 24bits
	int iRes = GetDIBits(hdc, hBitmap , 0, height,(LPVOID)buffer,&bmiInfo,DIB_RGB_COLORS);

	// release the device context
	ReleaseDC(NULL,hdc);

	if (!iRes)
	{
		delete buffer;
		return false;
	}

	return true;
}

// ** scan bitmap for transparent colors
bool CRegion::Scan(BYTE red, BYTE green, BYTE blue)
{
	BYTE tRed = red , tBlue = blue, tGreen = green;
	hRegion = CreateRectRgn(0, 0, width, height);
	if (!hRegion) return false;

	DWORD p=0;
	HRGN hTmpRgn;

	for (WORD y = 0; y < height; y++)
	{
		for (WORD x = 0; x < width; x++)
		{
			BYTE tempRed   = buffer[p+2];
			BYTE tempGreen = buffer[p+1];
			BYTE tempBlue  = buffer[p+0];

			if (tempRed == tRed && tempGreen == tGreen && tempBlue == tBlue)
			{
				hTmpRgn = CreateRectRgn(x,y,x+1,y+1);
				CombineRgn(hRegion, hRegion, hTmpRgn, RGN_XOR);
				DeleteObject(hTmpRgn);
			}
		// next pixel
		p+=3;
		}
	}

	return true;
}


// ** save region to disk
bool CRegion::Save(LPSTR filename)
{
	// get size
	int iSize = GetRegionData(hRegion, 0,0);

	// allocate space for it
	pData = new RGNDATA[iSize];
	if (!pData) return false;

	// get the region as binary data
	GetRegionData(hRegion, iSize, pData);

	// create file
	HANDLE hFile = CreateFile(filename, GENERIC_READ | GENERIC_WRITE,
						FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, CREATE_ALWAYS, 0, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
	{
		delete pData;
		return false;
	}

	// write region to file
	DWORD byteswritten;
	WriteFile(hFile, pData, iSize, &byteswritten, NULL);
	if (iSize != byteswritten)
	{
		CloseHandle(hFile);
		return false;
	}

	// write width and height to file
	WriteFile(hFile, (LPVOID)width, 4, &byteswritten, NULL);
	WriteFile(hFile, (LPVOID)height, 4, &byteswritten, NULL);

	CloseHandle(hFile);

	//delete pData;

	this->filename = filename;
	return true;
}


