// :: Part of RegionCreator
// Author: Sunshine

#include <windows.h>
#include <commctrl.h>
#include "CToolTip.h"


// ** constructor
CToolTip::CToolTip(HWND hwnd, HINSTANCE hInstance)
{
	this->hwnd = hwnd;
	this->hInstance = hInstance;

	// create tooltip window
	hToolTip = CreateWindowEx(WS_EX_TOPMOST, TOOLTIPS_CLASS, NULL,
								TTS_ALWAYSTIP | TTS_NOPREFIX | WS_POPUP,
								CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
								CW_USEDEFAULT, hwnd, 0, hInstance, NULL);
	if (hToolTip)
	{
		// tooltip window is topmost
		SetWindowPos(hToolTip, HWND_TOPMOST, 0, 0, 0, 0,
				SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	}
}


// ** destructor
CToolTip::~CToolTip()
{
	if (hToolTip)
		DestroyWindow(hToolTip);
}


// ** Add tooltip to a window
BOOL CToolTip::AddToolTip(HWND hWnd, LPSTR text)
{
	if (!hToolTip) return FALSE;

	TOOLINFO	ti;
	RECT		rect;

	if ( (hWnd ) && (GetClientRect(hWnd, &rect)) )
	{
		ZeroMemory(&ti, sizeof(TOOLINFO));

		ti.cbSize   = sizeof(TOOLINFO);
		ti.uFlags   = TTF_SUBCLASS | TTF_IDISHWND;
		ti.hwnd     = hWnd;
		ti.uId      = (INT_PTR)hWnd;
		ti.rect     = rect;
		ti.hinst    = hInstance;
		ti.lpszText = text;

		if (SendMessage(hToolTip,TTM_ADDTOOL,0,(LPARAM)&ti))
			return TRUE;
		else return FALSE;
	}
	return FALSE;
}


BOOL CToolTip::UpdateToolTip(HWND hWnd, LPSTR text)
{
	if (!hToolTip) return FALSE;

	TOOLINFO	ti;

	if(hWnd)
	{
		ZeroMemory(&ti,sizeof(TOOLINFO));

		ti.cbSize   = sizeof(TOOLINFO);
		ti.hwnd     = hWnd;
		ti.uId      = (int)hWnd;
		ti.hinst    = hInstance;
		ti.lpszText = text;

		if (SendMessage(hToolTip, TTM_UPDATETIPTEXT, 0, (LPARAM)&ti))
			return TRUE;
		else return FALSE;
	}
	return FALSE;
}