// :: Part of RegionCreator
// Author: Sunshine

#include "ImageBox.h"


LPSTR szImageBoxClsName="SunImageBox";
int	ImageBoxWidth = 415;
int ImageBoxHeight = 335;
int restH = 0;
int restV = 0;

// ** initialize imagebox
void InitImageBox()
{
	// Register imagebox class
    WNDCLASSEX wc;

    wc.cbSize         = sizeof(wc);
    wc.lpszClassName  = szImageBoxClsName;
    wc.hInstance      = GetModuleHandle(0);
	wc.lpfnWndProc    = ImageBox::WndProc;
    wc.hCursor        = LoadCursor (NULL, IDC_ARROW);
    wc.hIcon          = 0;
    wc.lpszMenuName   = 0;
    wc.hbrBackground  = (HBRUSH)GetSysColorBrush(COLOR_BTNFACE);
    wc.style          = 0 ;
    wc.cbClsExtra     = 0;
    wc.cbWndExtra     = sizeof( ImageBox * );
    wc.hIconSm        = 0;

	RegisterClassEx(&wc);

}


// ** create imageBox, returns handle to it
HWND CreateImageBox(HWND hwndParent, int x, int y)
{
    HWND hwndCtrl;

	int iVertBarWidth = GetSystemMetrics(SM_CXVSCROLL);
	int iHorzBarHeight = GetSystemMetrics(SM_CYHSCROLL);

    // create window
	hwndCtrl = CreateWindowEx(
                 WS_EX_CLIENTEDGE, // give it a standard border
                 szImageBoxClsName,
                 "SunImageBoxWin",
                 WS_VISIBLE | WS_CHILD | WS_HSCROLL | WS_VSCROLL,
                 x, y, ImageBoxWidth + iVertBarWidth, ImageBoxHeight + iHorzBarHeight,
                 hwndParent,
                 NULL, GetModuleHandle(0), NULL
               );


    return hwndCtrl;
}

// ** Get a state structure from the specified window
ImageBox * GetImageBoxCtrl(HWND hwnd)
{
    return (ImageBox *)GetWindowLong(hwnd, 0);
}



// ** Set a state structure for the specified window
void SetImageBoxCtrl(HWND hwnd, ImageBox *ccp)
{
    SetWindowLong(hwnd, 0, (LONG)ccp);
}

// ** constructor
ImageBox::ImageBox(HWND hwndParent)
{
	// save parent handle
	this->hwnd = hwndParent;
	// set variables to standard values
	image = NULL;
	iVertScrollPos = 0;
	iHorzScrollPos = 0;
	iMouseDownY = 0;
	iMouseDownY = 0;
	bMouseDown = false;
	bImageIsSmaller = false;
	fZoom = 1.0f;
	bImageLoaded = false;
}

// ** destructor
ImageBox::~ImageBox()
{

}

// ** set image file
bool ImageBox::setImage(CImageFile* ImageFile)
{
	if (ImageFile == NULL) return false;
	this->image = ImageFile;

	fZoom = 1.0f;

	// update scrollbars
	SCROLLINFO	si;
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_RANGE | SIF_PAGE;
	si.nMin = 0;
	restV = (image->getHeight() - ImageBoxHeight) % 5;
	si.nMax = max(image->getHeight() - ImageBoxHeight + restV, 0);
	si.nPage = 0;//si.nMax / 10;
	si.nPos = 0;
	SetScrollInfo(hwnd, SB_VERT,&si, TRUE);

	restH = (image->getWidth() - ImageBoxWidth) % 5;
	si.nMax = max(image->getWidth() - ImageBoxWidth + restH, 0);
	si.nPage = 0;//si.nMax / 10;
	SetScrollInfo(hwnd, SB_HORZ,&si, TRUE);

	iVertScrollPos = 0;
	iHorzScrollPos = 0;

	// update info variables
	bImageLoaded = true;
	if ((this->image->getHeight() < ImageBoxHeight) ||
		(this->image->getWidth() < ImageBoxWidth))
		bImageIsSmaller = true;
	else
		bImageIsSmaller = false;

	return true;
}


/*void ImageBox::ShowScrollBars(bool enabled)
{

	LONG lStyle = GetWindowLong(hwnd, GWL_STYLE);
	if (enabled)
		lStyle = lStyle | WS_HSCROLL;
	else
		lStyle = lStyle & WS_VSCROLL & WS_HSCROLL;

	LONG newStyle = SetWindowLongPtr(hwnd, GWL_STYLE, lStyle | 0x10000);
	//SetImageBoxCtrl(hwnd, this);

	newStyle = newStyle;
	InvalidateRect(hwnd,NULL, FALSE);
}*/


// ** message handler for imagebox
LRESULT CALLBACK ImageBox::WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    // retrieve the custom structure POINTER for THIS window
    ImageBox *ccp = GetImageBoxCtrl(hwnd);

    switch(msg)
    {

    // Allocate a new CustCtrl class for this window.
    case WM_NCCREATE:
        ccp = new ImageBox(hwnd);
        SetImageBoxCtrl(hwnd, ccp);
        return (ccp != NULL);

	// redirect messages
	case WM_CREATE:
		return ccp->OnCreate(wParam, lParam);

	case WM_VSCROLL:
		return ccp->OnVScroll(wParam, lParam);

	case WM_HSCROLL:
		return ccp->OnHScroll(wParam, lParam);

	case WM_LBUTTONDOWN:
		return ccp->OnLeftButDown(wParam, lParam);

	case WM_MOUSEMOVE:
		return ccp->OnMouseMove(wParam, lParam);

	case WM_LBUTTONUP:
		return ccp->OnLeftButUp(wParam, lParam);

	case WM_ERASEBKGND:
		return ccp->OnERASEBKGND(wParam, lParam);

	case WM_PAINT:
		return ccp->OnPaint(wParam, lParam);

    // Clean up when the window is destroyed.
    case WM_NCDESTROY:
        delete ccp;
        break;



	default: break;
    }

    return DefWindowProc(hwnd, msg, wParam, lParam);
}

// ** avoid flickering
LRESULT ImageBox::OnERASEBKGND  (WPARAM wParam, LPARAM lParam)
{
	return 1;
}


// ** Not used any more
LRESULT ImageBox::OnCreate(WPARAM wParam, LPARAM lParam)
{
	return 0;
}


LRESULT ImageBox::OnVScroll(WPARAM wParam, LPARAM lParam)
{
	SCROLLINFO	si;

	si.cbSize = sizeof(SCROLLINFO);
	si.fMask  = SIF_ALL;
	GetScrollInfo (hwnd, SB_VERT, &si) ;

	switch (LOWORD(wParam))
	{
	case SB_LINEDOWN:
		si.nPos += 5 ;
		break;

	case SB_LINEUP:
		si.nPos -= 5;
		break;

	case SB_PAGEUP:
		si.nPos = max(0, si.nPos -  (si.nMax / 10)); ;
		break ;

	case SB_PAGEDOWN:
		si.nPos = min(si.nMax, si.nPos +  (si.nMax / 10));
		break ;

	case SB_THUMBTRACK:
		si.nPos = HIWORD(wParam);


	default:break;
	}

	si.fMask = SIF_POS ;
	// save vertical scrollbar position
	iVertScrollPos = si.nPos;
	SetScrollInfo (hwnd, SB_VERT, &si, TRUE) ;
	GetScrollInfo (hwnd, SB_VERT, &si) ;

	InvalidateRect(hwnd, NULL, TRUE);

	return 0;
}


LRESULT ImageBox::OnHScroll(WPARAM wParam, LPARAM lParam)
{
	SCROLLINFO	si;

	si.cbSize = sizeof(SCROLLINFO);
	si.fMask  = SIF_ALL;
	GetScrollInfo (hwnd, SB_HORZ, &si) ;

	switch (LOWORD(wParam))
	{
	case SB_LINEDOWN:
		si.nPos += 2;
		break;

	case SB_LINEUP:
		si.nPos -= 2;
		break;

	case SB_PAGEUP:
		si.nPos = max(0, si.nPos -  (si.nMax / 10));
		break ;

	case SB_PAGEDOWN:
		si.nPos = min(si.nMax, si.nPos +  (si.nMax / 10));
		break ;

	case SB_THUMBTRACK:
		si.nPos = HIWORD(wParam);
		break;


	default:break;
	}

	si.fMask = SIF_POS ;
	// save horizontal scrollbar position
	iHorzScrollPos = si.nPos;
	int t = SetScrollInfo (hwnd, SB_HORZ, &si, TRUE) ;
	GetScrollInfo (hwnd, SB_HORZ, &si) ;
	InvalidateRect(hwnd, NULL, TRUE);

	return 0;
}


LRESULT ImageBox::OnLeftButDown (WPARAM wParam, LPARAM lParam)
{
	if (bImageIsSmaller) return 0;
	// save cursor position when mouse button is pressed
	iMouseDownX = LOWORD(lParam);
	iMouseDownY = HIWORD(lParam);

	bMouseDown = true;
	return 0;
}

LRESULT ImageBox::OnMouseMove (WPARAM wParam, LPARAM lParam)
{
	if ((bImageIsSmaller) || (!this->bImageLoaded)) return 0;
	HCURSOR hCross = LoadCursor(NULL , IDC_CROSS);
	SetCursor(hCross);

	if ( (bMouseDown) && (wParam == MK_LBUTTON) )
	{
		iVertScrollPos = iVertScrollPos + (iMouseDownY - HIWORD(lParam)) ;
		iHorzScrollPos = iHorzScrollPos + (iMouseDownX - LOWORD(lParam));

		if (iHorzScrollPos < 0) iHorzScrollPos = 0;
		if (iHorzScrollPos > abs((image->getWidth() - ImageBoxWidth)*fZoom))
			iHorzScrollPos = abs((image->getWidth() - ImageBoxWidth+ restH )*fZoom );

		if (iVertScrollPos < 0) iVertScrollPos = 0;
		if (iVertScrollPos > abs((image->getHeight() - ImageBoxHeight)*fZoom))
			//iVertScrollPos = abs((image->getHeight() - ImageBoxHeight)*fZoom + restV);
			iVertScrollPos = abs((image->getHeight() - ImageBoxHeight + restV)*fZoom);

		iMouseDownX = LOWORD(lParam);
		iMouseDownY = HIWORD(lParam);

		// update scrollbars
		SCROLLINFO	si;
		si.cbSize = sizeof(SCROLLINFO);
		si.fMask  = SIF_ALL;
		GetScrollInfo (hwnd, SB_HORZ, &si) ;
		si.nPos = iHorzScrollPos;
		SetScrollInfo (hwnd, SB_HORZ, &si, TRUE) ;
		GetScrollInfo (hwnd, SB_VERT, &si) ;
		si.nPos = iVertScrollPos;
		SetScrollInfo (hwnd, SB_VERT, &si, TRUE) ;

		InvalidateRect(hwnd, NULL, TRUE);
	}

	return 0;
}

LRESULT ImageBox::OnLeftButUp	  (WPARAM wParam, LPARAM lParam)
{
	if (bImageIsSmaller) return 0;
	bMouseDown = false;
	return 0;
}

LRESULT ImageBox::OnPaint(WPARAM wParam, LPARAM lParam)
{
	HDC				hdc;
    PAINTSTRUCT		ps;

	hdc = BeginPaint(hwnd, &ps);

	if ( (image != NULL) && (!IsBadReadPtr(image, sizeof(CImageFile))) )
	{
		int xPart = min(iHorzScrollPos, (int)(image->getWidth()*fZoom));
		xPart = max(0, xPart);
		int yPart = min(iVertScrollPos, (int)(image->getHeight()*fZoom));
		yPart = max(0, yPart);

		if ((image->bFileLoaded) )
			//image->DrawEx(hdc, 0, 0, iHorzScrollPos, iVertScrollPos, 1.0f);
			image->DrawExZoom(hdc, 0, 0, ImageBoxWidth, ImageBoxHeight,
							iHorzScrollPos/fZoom, iVertScrollPos/fZoom, fZoom);

	}

	EndPaint(hwnd, &ps);
	return 0;

}

bool ImageBox::setZoom(float fZoom)
{
	this->fZoom = fZoom;


	// erase background
	InvalidateRect(hwnd, NULL, false);
	HDC				hdc;
    PAINTSTRUCT		ps;
	RECT			rect;
	hdc = BeginPaint(hwnd, &ps);
	HBRUSH brush = (HBRUSH)GetStockObject(WHITE_BRUSH);//RGB(192,192,192));
	GetClientRect(hwnd, &rect);
	int t = FillRect(hdc, &rect, brush);
	DeleteObject(brush);
	EndPaint(hwnd, &ps);
	InvalidateRect(hwnd, NULL, false);


	// update scrollbars
	SCROLLINFO	si;
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_RANGE | SIF_PAGE;
	si.nMin = 0;
	restV = (image->getHeight() - ImageBoxHeight) % 5;
	if ( (image->getHeight()*fZoom) - ImageBoxHeight > 0)
		si.nMax = max(abs((image->getHeight() - ImageBoxHeight + restV)*fZoom), 0);
	else si.nMax = 0;
	si.nPage = 0;
	si.nPos = 0;
	SetScrollInfo(hwnd, SB_VERT,&si, TRUE);

	restH = (image->getWidth() - ImageBoxWidth) % 5;
	if ( (image->getWidth()*fZoom) - ImageBoxWidth > 0)
		si.nMax = max(abs((image->getWidth() - ImageBoxWidth + restH)*fZoom), 0);
	else si.nMax = 0;
	si.nPage = 0;
	si.nPos = 0;
	SetScrollInfo(hwnd, SB_HORZ,&si, TRUE);

	iVertScrollPos = 0;
	iHorzScrollPos = 0;

	// update info variables
	bImageLoaded = true;
	if ((this->image->getHeight()*fZoom < ImageBoxHeight) ||
		(this->image->getWidth()*fZoom < ImageBoxWidth))
		bImageIsSmaller = true;
	else
		bImageIsSmaller = false;

	return true;
}
