/*=====================================================================================
** Sunshine's Region Creator
** -------------------------
** This little program scans images for a transparent color and save the region as a
** file. See readme.txt for further information.
**=====================================================================================*/

// ** includes
#include <windows.h>
#include "CImageFile.h"
#include "CRegion.h"
#include "ImageBox.h"
#include "CToolTip.h"
#include "testWindow.h"
#include "CSunRegion.h"
#include "resource.h"

// ** Forward declarations
BOOL AboutBoxProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);

// ** Global Variables
	HINSTANCE		hInst;
	char			filename[512];
	LPSTR			filterstring = "Image Files (*.bmp,*.jpg,*.gif,*.png)\0"
								   "*.bmp;*.jpg;*.jpeg;*.gif\0"
								   "All Files (*.*)\0*.*\0\0";

	LPSTR			filterstringForSave = "Region File (*.rgn)\0*.rgn\0\0";
	char			filenameSave[MAX_PATH];

	HWND			hImageBox;				// handle to imagebox
	ImageBox*		imageBox;				// instance of imagebox
	CImageFile*		imageFile;				// image file to load
	CImageFile*		backgroundImage;		// background image
	CRegion*		rgn;					// region to be created	
	CToolTip*		ToolTip;				// tooltips for buttons
	HWND			hZoomBox;				// combobox for choosing zoom

	const			WindowWidth = 600;		// width of main window
	const			WindowHeight = 480;		// height of main window

	bool			bPickColorMode = false;		// are we in color pick mode?
	HCURSOR			hCross;				// cross cursor for color picking

	COLORREF		color;				// transparent color
	COLORREF		oldColor;			// old transparent color
	float			fZoom = 1.0f;		// zoom factor



// ** Main dialog message loop
BOOL DlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{

	switch(msg)
	{

	// ** initiatialize window
	case WM_INITDIALOG:
	{
		// center window
		int x = GetSystemMetrics(SM_CXSCREEN);
		int y = GetSystemMetrics(SM_CYSCREEN); 
		MoveWindow(hwnd, (x / 2) - (WindowWidth / 2),
					(y / 2) - (WindowHeight / 2),
					WindowWidth, WindowHeight, true);

		
		// load images on buttons
		HANDLE hBitmap = LoadImage(hInst, MAKEINTRESOURCE(ID_OPENIMAGE), IMAGE_BITMAP,
									0, 0 , LR_DEFAULTCOLOR);
		SendMessage(GetDlgItem(hwnd, ID_OPEN), BM_SETIMAGE, 
					IMAGE_BITMAP, (LPARAM)hBitmap);

		hBitmap = LoadImage(hInst, MAKEINTRESOURCE(ID_SCANIMAGE), IMAGE_BITMAP,
									0, 0 , LR_DEFAULTCOLOR);
		SendMessage(GetDlgItem(hwnd, ID_SCAN), BM_SETIMAGE, 
					IMAGE_BITMAP, (LPARAM)hBitmap);

		hBitmap = LoadImage(hInst, MAKEINTRESOURCE(ID_SAVEIMAGE), IMAGE_BITMAP,
									0, 0 , LR_DEFAULTCOLOR);
		SendMessage(GetDlgItem(hwnd, ID_SAVE), BM_SETIMAGE, 
					IMAGE_BITMAP, (LPARAM)hBitmap);

		hBitmap = LoadImage(hInst, MAKEINTRESOURCE(ID_TESTIMAGE), IMAGE_BITMAP,
									0, 0 , LR_DEFAULTCOLOR);
		SendMessage(GetDlgItem(hwnd, ID_TESTWIN), BM_SETIMAGE, 
					IMAGE_BITMAP, (LPARAM)hBitmap);

		// disable some buttons
		EnableWindow(GetDlgItem(hwnd, ID_SCAN), false);
		EnableWindow(GetDlgItem(hwnd, ID_SAVE), false);
		EnableWindow(GetDlgItem(hwnd, ID_TESTWIN), false);

		// initialize imagebox
		InitImageBox();
		hImageBox = CreateImageBox(hwnd, 10, 90);
		imageBox = GetImageBoxCtrl(hImageBox);

		// load cursor
		hCross = LoadCursor(NULL , IDC_CROSS);
		HCURSOR hCurPrev = SetCursor(hCross);

		// set inital color values in edit boxes
		SetDlgItemInt(hwnd, ID_REDEDIT, 255, false);
		SetDlgItemInt(hwnd, ID_GREENEDIT, 255, false);
		SetDlgItemInt(hwnd, ID_BLUEEDIT, 255, false);

		// add tooltips
		ToolTip = new CToolTip(hwnd, hInst);
		ToolTip->AddToolTip(GetDlgItem(hwnd, ID_OPEN), 
							"Open image file...");
		ToolTip->AddToolTip(GetDlgItem(hwnd, ID_SAVE),
							"Save region...");
		ToolTip->AddToolTip(GetDlgItem(hwnd, ID_SCAN),
							"Scan image for creating a region...");
		ToolTip->AddToolTip(GetDlgItem(hwnd, ID_TESTWIN),
							"Test region...");

		// initialize zoom combobox
		hZoomBox = GetDlgItem(hwnd, ID_COMBOZOOM);
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("50%"));
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("75%"));
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("100%"));
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("120%"));
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("150%"));
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("200%"));
		SendMessage(hZoomBox, CB_ADDSTRING, 0, (LPARAM)("300%"));
		SendMessage(hZoomBox, CB_SETCURSEL, 2, 0);
		MoveWindow(hZoomBox, 60, 57, 68, 100, FALSE);

		// load and assign region to main window
		CSunRegion* region = new CSunRegion(hwnd);
		//region->LoadFromFile("rgnCreator.rgn");
		region->LoadFromRes(MAKEINTRESOURCE(ID_MAINRGN), "CUSTOM");
		delete region;

		// load background file
		backgroundImage = new CImageFile();
		backgroundImage->Load("C:\\RegionCreatorSkin.bmp");
		InvalidateRect(hwnd, NULL ,TRUE);

		// set window caption
		SetWindowText(hwnd, "Region Creator 1.0");

		SetClassLong(hwnd,          // window handle 
					GCL_HICON,              // changes icon 
					(LONG) LoadIcon(hInst, MAKEINTRESOURCE(ID_MAINICON)));
	}
	break;

	// ** Timer messages, only active while picking a color
	case WM_TIMER:
	{
		// set cross cursor
		SetCursor(hCross);
		
		// get color and update color edit boxes
		POINT	pt;
		HDC hdc = GetDC(hwnd);
		GetCursorPos(&pt); 
		ScreenToClient(hwnd, &pt); 
		color = GetPixel(hdc, pt.x, pt.y);
		if (color == CLR_INVALID) return FALSE;
		SetDlgItemInt(hwnd, ID_REDEDIT, GetRValue(color), false);
		SetDlgItemInt(hwnd, ID_GREENEDIT, GetGValue(color), false);
		SetDlgItemInt(hwnd, ID_BLUEEDIT, GetBValue(color), false);

		// paint little rect with chosen color
		HBRUSH hbrBkgnd = CreateSolidBrush(color);
		RECT rect;
		rect.left = 290; rect.top = 50;
		rect.right = 352; rect.bottom = 70;
		FillRect(hdc, &rect, hbrBkgnd);
		DeleteObject(hbrBkgnd); 

		// show info text
		LPSTR szTemp = "Click left to choose color, click right to abort...";
		SetDlgItemText(hwnd, ID_INFOTEXT, szTemp);


	}
	break;


	case WM_PAINT:
		{
			// draw background image
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			backgroundImage->Draw(hdc, 0, 0);
			EndPaint(hwnd, &ps);
		}
		break;


	// ** paint dialog and colors white
	case WM_CTLCOLORDLG:
	case WM_CTLCOLORSTATIC:
	case WM_CTLCOLORBTN:
	{
          HBRUSH hb = CreateSolidBrush(RGB(255,255,255));
          SetBkColor((HDC)wparam, RGB(255,255,255));
          return (int)(GetStockObject(WHITE_BRUSH));
	}
	break;


	// ** Right mouse button is clicked
	case WM_RBUTTONDOWN:
	{
		// abort color pick mode
		if (bPickColorMode)
		{
			HDC hdc = GetDC(hwnd);
			bPickColorMode = false;
			KillTimer(hwnd, 99);
			ReleaseCapture();

			// restore old color
			SetDlgItemInt(hwnd, ID_REDEDIT, GetRValue(oldColor), false);
			SetDlgItemInt(hwnd, ID_GREENEDIT, GetGValue(oldColor), false);
			SetDlgItemInt(hwnd, ID_BLUEEDIT, GetBValue(oldColor), false);

			// paint little rect with old color
			HBRUSH hbrBkgnd = CreateSolidBrush(oldColor);
			RECT rect;
			rect.left = 290; rect.top = 50;
			rect.right = 352; rect.bottom = 70;
			FillRect(hdc, &rect, hbrBkgnd);
			DeleteObject(hbrBkgnd); 
			SetDlgItemText(hwnd, ID_INFOTEXT, "");
		}
	}
	break;


	// ** Left mouse button is clicked
	case WM_LBUTTONDOWN:
	{
		// end color pick mode
		if (bPickColorMode) 
		{
			bPickColorMode = false;
			KillTimer(hwnd, 99);
			ReleaseCapture();
		}
		else
		{
			// draw main window
			ReleaseCapture();
			SendMessage(hwnd, WM_SYSCOMMAND, 0xf012, 0);
		}
		SetDlgItemText(hwnd, ID_INFOTEXT, "");
	}
	break;

	
	// ** Bye bye...
	case WM_CLOSE:
	case WM_DESTROY:
	  {
		  EndDialog(hwnd, 0);
		  PostQuitMessage(0);
	  }
	  break;

	
	
	case WM_COMMAND:
	{
		switch(LOWORD(wparam))
		{
		case ID_OPEN:
		{
			// open imagefile
			OPENFILENAME ofn;

			ZeroMemory(&ofn, sizeof(OPENFILENAME));
			ofn.lStructSize = sizeof(OPENFILENAME);	
			ofn.hwndOwner = hwnd;	
			ofn.hInstance = hInst;	
			ofn.lpstrFilter = filterstring;	
			ofn.lpstrFile = filename;	
			ofn.nMaxFile =  512;
			ofn.lpstrTitle = "Choose file...";	
			ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_LONGNAMES | OFN_EXPLORER | OFN_HIDEREADONLY;	
			if (GetOpenFileName(&ofn))
			{
				InvalidateRect(hImageBox, NULL, TRUE);
				// load image file
				if (imageFile != NULL)
					delete imageFile;
				imageFile = new CImageFile();
				imageFile->Load(filename);
				// set zoom back to 100%
				imageBox->setImage(imageFile);
				fZoom = 1.0f;
				//imageBox->setZoom(fZoom);
				SendMessage(hZoomBox, CB_SETCURSEL, 2, 0);
				
				
				
				
				// enable some buttons and show loaded image
				EnableWindow(GetDlgItem(hwnd, ID_SCAN), true);
				EnableWindow(GetDlgItem(hwnd, ID_SAVE), false);
				EnableWindow(GetDlgItem(hwnd, ID_TESTWIN), false);
				InvalidateRect(hImageBox, NULL, FALSE);
				
			}
		}
		break;

		case ID_SAVE:
		{
			// save imagefile
			OPENFILENAME ofn;

			ZeroMemory(&ofn, sizeof(OPENFILENAME));
			ofn.lStructSize = sizeof(OPENFILENAME);	
			ofn.hwndOwner = hwnd;	
			ofn.hInstance = hInst;	
			ofn.lpstrFilter = filterstringForSave;	
			ofn.lpstrFile = filenameSave;	
			ofn.nMaxFile =  512;
			ofn.lpstrTitle = "Choose file for saving...";	
			if (GetSaveFileName(&ofn))
			{	
				
				// check for file extension
				int len = lstrlen(filenameSave);
				bool bTemp;
				if (!( (filenameSave[len-1] == 'n') && (filenameSave[len-2] == 'g') &&
					 (filenameSave[len-3] == 'r') && (filenameSave[len-4] == '.') ))
				{
					// no extension, so append it
					LPSTR szTemp = (LPSTR)LocalAlloc(LMEM_FIXED, len+4);
					lstrcpy(szTemp, filenameSave);
					lstrcat(szTemp, ".rgn");
					bTemp = rgn->Save(szTemp);
					//LocalFree(szTemp);
				}
				else
					bTemp = rgn->Save(filenameSave);

				// check if error occured while saving
				if (!bTemp)
					MessageBox(hwnd, "File could not be saved! Make sure you have access rights \n "
									 "and file is not opened by another application.", "Error....", MB_OK);
				else
					SetDlgItemText(hwnd, ID_INFOTEXT, "File saved succesfully!");	

				
			}
		}
		break;


		// ** Scan image and create region
		case ID_SCAN:
		{
			if (!imageFile->bFileLoaded)
			{
				MessageBox(hwnd, "No bitmap is loaded!", "Error", MB_OK);
				break;
			}

			// get transparent color
			BOOL b1, b2, b3;
			BYTE red = GetDlgItemInt(hwnd, ID_REDEDIT, &b1, FALSE);
			BYTE green = GetDlgItemInt(hwnd, ID_GREENEDIT, &b2, FALSE);
			BYTE blue = GetDlgItemInt(hwnd, ID_BLUEEDIT, &b3, FALSE);
			if ( (!b1) || (!b2) || (!b3) )
			{
				MessageBox(hwnd, "Please enter values between 0 and 255 in all three boxes!",
								"Error", MB_OK);
				break;
			}

			SetDlgItemText(hwnd, ID_INFOTEXT, "PLEASE WAIT WHILE IMAGE IS SCANNED..."); 
			// create region
			rgn = new CRegion(imageFile->getBitmap());
			rgn->Convert();
			rgn->Scan(red, green, blue);
			
			EnableWindow(GetDlgItem(hwnd, ID_SAVE), true);
			EnableWindow(GetDlgItem(hwnd, ID_TESTWIN), true);
			SetDlgItemText(hwnd, ID_INFOTEXT, "");
		}
		break;

		case ID_TESTWIN:
		{
			// Show test window
			CreateTestWin(rgn);
		}
		break;

		case ID_PICKCOLOR:
		{
			// save old color
			BOOL bTemp;
			BYTE red = GetDlgItemInt(hwnd, ID_REDEDIT, &bTemp, FALSE);
			if (!bTemp) red = 0;
			BYTE green = GetDlgItemInt(hwnd, ID_GREENEDIT, &bTemp, FALSE);
			if (!bTemp) green = 0;
			BYTE blue = GetDlgItemInt(hwnd, ID_BLUEEDIT, &bTemp, FALSE);
			if (!bTemp) blue = 0;
			oldColor = RGB(red, green, blue);
			
			// enable color pick mode
			bPickColorMode = true;
			hCross = LoadCursor(NULL , IDC_CROSS);
			SetCursor(hCross);
			SetTimer(hwnd, 99, 10, NULL);
			SetCapture(hwnd);
		}
		break;

		case ID_COMBOZOOM:
		{
			// new zoomfactor chosen?
			if (HIWORD(wparam) == CBN_SELENDOK)
			{
				int iIndex = (int)SendMessage(hZoomBox, CB_GETCURSEL, 0, 0);
				switch(iIndex)
				{
				case 0: fZoom = 0.5f; break;
				case 1: fZoom = 0.75f; break;
				case 2: fZoom = 1.0f; break;
				case 3: fZoom = 1.2f; break;
				case 4: fZoom = 1.5f; break;
				case 5: fZoom = 2.0f; break;
				case 6: fZoom = 3.0f; break;
				default: break;
				}
				imageBox->setZoom(fZoom);
			}
		}
		break;

		// ** Bye bye
		case ID_X:
		{
			SendMessage(hwnd, WM_CLOSE, 0, 0);
		}
		break;

		// ** Minimize window
		case ID_V:
		{
			ShowWindow(hwnd,SW_MINIMIZE);
		}
		break;

		// ** About Box
		case ID_ABOUT:
			{
				DialogBoxParamA(hInst, MAKEINTRESOURCE(102), NULL, 
								(DLGPROC)AboutBoxProc, NULL);
			}
			break;

		default:break;
		}
	}
	break;

	default:break;

  }

  return FALSE;
}

// ** Application entrypoint
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow)
{
	hInst = GetModuleHandle(0);
	DialogBoxParamA(hinstance, MAKEINTRESOURCE(101), NULL, (DLGPROC)DlgProc, NULL);

	return 0;
}




// ************************ About Box Stuff ********************************

HANDLE hAboutBackground;
// ** Main aboutbox message loop
BOOL AboutBoxProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{

	switch(msg)
	{
	case WM_DESTROY:
	case WM_CLOSE:
		{
			EndDialog(hwnd, 0);
		}
		break;

	case WM_LBUTTONDOWN:
		{
			ReleaseCapture();
			SendMessage(hwnd, WM_SYSCOMMAND, 0xf012, 0);
		}
		break;

	case WM_INITDIALOG:
		{
			// center window
			int x = GetSystemMetrics(SM_CXSCREEN);
			int y = GetSystemMetrics(SM_CYSCREEN); 
			int WindowWidth = 300;
			int WindowHeight = 300;
			MoveWindow(hwnd, (x / 2) - (WindowWidth / 2),
						(y / 2) - (WindowHeight / 2),
						WindowWidth, WindowHeight, true);

			// load background bitmap
			hAboutBackground = LoadImage(hInst, MAKEINTRESOURCE(ID_ABOUTIMAGE), IMAGE_BITMAP,
										0, 0 , LR_DEFAULTCOLOR);

			// load and assign region to main window
			CSunRegion* region = new CSunRegion(hwnd);
			region->LoadFromRes(MAKEINTRESOURCE(ID_ABOUTRGN), "CUSTOM");
			delete region;

			SetWindowText(hwnd, "About Region Creator");

			
		}
		break;

	case WM_PAINT:
		{
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(hwnd, &ps);
			HDC memDC = CreateCompatibleDC(hdc);
			HBITMAP hOldBitmap = (HBITMAP)SelectObject(memDC, (HBITMAP)hAboutBackground);
			// Display the bitmap into window
			BITMAP bm;
			GetObject((HBITMAP)hAboutBackground, sizeof(BITMAP), &bm);
			BitBlt(hdc, 0, 0, bm.bmWidth, bm.bmHeight, memDC, 0, 0, SRCCOPY);
			// Delete the temporary bitmap DC
			SelectObject(memDC, hOldBitmap);
			DeleteDC(memDC);
		}
		break;

	case WM_COMMAND:
		{
			switch (wparam)
			{
			case ID_ABOUTCLOSE:
				{
					EndDialog(hwnd, 0);
				}
				break;

			default:break;
			}
		}
		break;

		

	default:break;

	}

  return FALSE;
}

