// :: Part of RegionCreator
// :: Author: Sunshine

#include "testWindow.h"
#include "CImageFile.h"
#define ID_OPENIMAGE    210
#define ID_CLOSEWIN    212

namespace testwin
{
	HINSTANCE		hInst;			// instance of application
	HWND			hwnd;			// handle of testwindow
	CRegion*		rgn;			// pointer to our region
	int				iSize;			// size of region in bytes
	HMENU			menu;			// handle to popupmenu
	char			filename[512];	// filename buffer to load an image
	LPSTR			filterstring = "Image Files (*.bmp,*.jpg,*.gif,*.png)\0"
									"*.bmp;*.jpg;*.jpeg;*.gif\0\0";
	CImageFile*		imageFile;		// pointer to image file
	LPRGNDATA		pData = NULL;	// region as buffer

}


// ** message handler for testwindow
LRESULT WINAPI TestWndProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{

	switch(msg)
    {
        case WM_DESTROY:
		case WM_CLOSE:
		{
            DestroyWindow(hwnd);
            PostQuitMessage(0);
            return 0;
		}
		break;

		case WM_RBUTTONDOWN:
		{
			POINT pt;
			pt.x = LOWORD(lParam);
			pt.y = HIWORD(lParam);
			ClientToScreen(hwnd, &pt);

			TrackPopupMenuEx(testwin::menu, TPM_LEFTALIGN | TPM_TOPALIGN,
				pt.x, pt.y, testwin::hwnd, NULL);
		}
		break;

		case WM_LBUTTONDOWN:
		{
			ReleaseCapture();
			SendMessage(hwnd, WM_SYSCOMMAND, 0xf012, 0);
		}
		break;

		case WM_CREATE:
		{
			// load & set region
			//int iSize = GetRegionData(testwin::rgn->hRegion, 0,0);

			// move region, so we don't see caption bar
			// not used any more, cause we specify WS_POPUP in CreateWindow
			/*
			XFORM xForm;
			xForm.eDx = GetSystemMetrics(SM_CXFRAME);
			xForm.eDy = GetSystemMetrics(SM_CYCAPTION)+GetSystemMetrics(SM_CYFRAME);
			xForm.eM11 = 1.0f; xForm.eM12 = 0.0f;
			xForm.eM21 = 0.0f; xForm.eM22 = 1.0f;
			*/

			
			HRGN hRegion = ExtCreateRegion(NULL, testwin::iSize, testwin::pData);
			SetWindowRgn( hwnd,hRegion, FALSE );

			testwin::menu = CreatePopupMenu();
			AppendMenu(testwin::menu, MF_STRING, ID_OPENIMAGE, "Open Image File...");
			AppendMenu(testwin::menu, MF_SEPARATOR, -1, "");
			AppendMenu(testwin::menu, MF_STRING, ID_CLOSEWIN, "Close window...");

			testwin::imageFile = NULL;
		}
		break;

		case WM_PAINT:
		{
			if (testwin::imageFile != NULL)
			{
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(hwnd, &ps);
				testwin::imageFile->Draw(hdc, 0, 0);
				EndPaint(hwnd, &ps);
			}
		}
		break;


		case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case ID_CLOSEWIN:

				DestroyWindow(hwnd);
				if (testwin::pData) delete testwin::pData;
				PostQuitMessage(0);
				return 0;
				break;

			case ID_OPENIMAGE:
				OPENFILENAME ofn;
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize = sizeof(OPENFILENAME);
				ofn.hwndOwner = hwnd;
				ofn.hInstance = testwin::hInst;
				ofn.lpstrFilter = testwin::filterstring;
				ofn.lpstrFile = testwin::filename;
				ofn.nMaxFile =  512;
				ofn.lpstrTitle = "Choose file for saving...";
				if (GetSaveFileName(&ofn))
				{
					if (testwin::imageFile != NULL)
						delete testwin::imageFile;
					testwin::imageFile = new CImageFile();
					testwin::imageFile->Load(testwin::filename);
					InvalidateRect(hwnd, NULL ,TRUE);
				}
				break;


			default:break;
			}
		}

    }
    return DefWindowProc( hwnd, msg, wParam, lParam );
}


bool CreateTestWin(CRegion* rgn)
{
	testwin::hInst = GetModuleHandle(0);
	testwin::rgn = rgn;
	// make a copy of our region
	// why? cause the call SetWindowRgn destroys our handle to our region
	testwin::iSize = GetRegionData(rgn->hRegion, 0,0);
	testwin::pData = new RGNDATA[testwin::iSize];
	if (!testwin::pData) return false;
	GetRegionData(rgn->hRegion, testwin::iSize, testwin::pData);



	WNDCLASSEX wc;
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_CLASSDC;
	wc.lpfnWndProc = TestWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = testwin::hInst;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "RGNTestWin";
	wc.hIconSm = NULL;
	if (RegisterClassEx(&wc) == 0)
	{
		MessageBox(0, "Could not register test window!", "Error", MB_OK);
		return false;
	}

	testwin::hwnd = CreateWindow("RGNTestWin", "Region Creator Test Window", WS_POPUP,
						0, 0, testwin::rgn->getWidth(), testwin::rgn->getHeight(),
						NULL, NULL, testwin::hInst, NULL);
	if (testwin::hwnd == NULL) return false;

	// show window
	ShowWindow(testwin::hwnd, SW_SHOWNORMAL);
	UpdateWindow(testwin::hwnd);

	bool done = false;
	MSG msg;

	while (!done)
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
		{
			if (msg.message == WM_QUIT)
			{
				done = true;
			}
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}

	}

	UnregisterClass("RGNTestWin", testwin::hInst);

	return true;
}
